@@ -2,6 +2,7 @@ package perl5i::2::Meta;
 
 use strict;
 use warnings;
+use v5.10.0;
 
 # Be very careful not to import anything.
 require Carp;
@@ -44,6 +45,44 @@ sub linear_isa {
     return wantarray ? @$isa : $isa;
 }
 
+sub methods {
+    my $self = shift;
+    my $opts = shift // {};
+    my $top = $self->class;
+
+    my %exclude;
+
+    state $defaults = {
+        with_UNIVERSAL       => 0,
+        just_mine            => 0,
+    };
+
+    $opts = { %$defaults, %$opts };
+    $exclude{UNIVERSAL} = !$opts->{with_UNIVERSAL};
+
+    my @classes = $opts->{just_mine} ? $self->class : $self->linear_isa;
+
+    my %all_methods;
+    for my $class (@classes) {
+        next if $exclude{$class} && $class ne $top;
+
+        my $sym_table = $class->mc->symbol_table;
+        for my $name (keys %$sym_table) {
+            next unless *{$sym_table->{$name}}{CODE};
+            $all_methods{$name} = $class;
+        }
+    }
+
+    return wantarray ? keys %all_methods : [keys %all_methods];
+}
+
+sub symbol_table {
+    my $self = shift;
+    my $class = $self->class;
+
+    no strict 'refs';
+    return \%{$class.'::'};
+}
 
 # A single place to put the "method not found" error.
 my $method_not_found = sub {
